/*
Copyright (c) 2009, Luke Berndt
All rights reserved.
 
 * More Information: http://www.tagflo.com

Redistribution and use in source and binary forms, with or without modification, are permitted 
provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions 
  and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
  and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of RoboTastic Software nor the names of its contributors may be used to endorse or 
  promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/


using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Forms;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;
using System.Data.SqlServerCe;
using System.Data.OleDb;
using System.Data.SqlClient;
using System.Data;
using System.Linq;
using System.Windows.Threading;

namespace TagFlo
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>

    public partial class ImportDialog : System.Windows.Window
    {
        private ImportWorker importWorker;
        ThreadedObservableCollection<Path> pathCollection;

        private TreeViewItem FindTreeViewItemChild(ItemCollection items, String childString)
        {
            foreach (TreeViewItem child in items)
            {
                if (child.Header.ToString() == childString)
                {
                    return child;
                }

            }
            return null;
        }

       

        private void Click_ScanDirectories(object sender, RoutedEventArgs e)
        {
            importWorker.AddScan();
        }

        private void RemoveDirectoryButton_Click(object sender, RoutedEventArgs e)
        {
            if (ImportTree.SelectedItem != null)
            {
                HierarchyPath path = (HierarchyPath)ImportTree.SelectedItem;

              
                importWorker.RemoveDirectory(path.Path.FullPath);
               }

        }
        private void Click_AddDirectory(object sender, RoutedEventArgs e)
        {
            FolderBrowserDialog dialog = new FolderBrowserDialog();

            if (dialog.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                String folderName = dialog.SelectedPath;
                importWorker.AddDirectory(folderName);
                //sqlDataSet.RefreshImportTable();
                //ImportTree.Items.Clear();
                //Perhaps add some animation when adding a directory.
            }

        }
        private void OnInit(object sender, EventArgs e)
        {
         
        }

        private Dispatcher dispatcherUIThread;
        private delegate void BuildFolderCallback();
            
        private void BuildFolderHierarchy()
        {
            if (dispatcherUIThread.CheckAccess())
            {
                var paths = pathCollection.ToList().AsPathHierarchy();// (p => p.Parent);
                PathTreeViewItem.ItemsSource = null;
                PathTreeViewItem.ItemsSource = paths;
                
            }
            else
            {
                dispatcherUIThread.Invoke(DispatcherPriority.Send, new BuildFolderCallback(BuildFolderHierarchy));
            }  

        }
        private void DirectoryChangeHandler(object sender, EventArgs e)
        {
            BuildFolderHierarchy();
        }

        public ImportDialog(MainWindow mainWindow)
        {

          
            //importWorker ;
            InitializeComponent();
            pathCollection = mainWindow.pathCollection;
            importWorker = mainWindow.importWorker;
            importWorker.DirectoryChanged += new EventHandler(DirectoryChangeHandler);


            dispatcherUIThread = mainWindow.Dispatcher;
            BuildFolderHierarchy();
         
        }

    }
}