/*
Copyright (c) 2009, Luke Berndt
All rights reserved.
 
 * More Information: http://www.tagflo.com

Redistribution and use in source and binary forms, with or without modification, are permitted 
provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions 
  and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
  and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of RoboTastic Software nor the names of its contributors may be used to endorse or 
  promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/


using System;
using System.Collections.Generic;
using System.Text;
using System.Data.SqlServerCe;
using System.Data.OleDb;
using System.Data.SqlClient;
using System.Data;
using System.Windows.Data;
using System.IO;
using System.Windows.Controls;
using System.Windows;
using System.Windows.Media;

namespace TagFlo
{
    class SqlTreeViewItem : TreeViewItem
    {
        private DataTable treeItemDataTable;
        private String _table;
        private String _column;

        public string Table
        {
            get
            {
                return _table;
            }
        }

        public string Column
        {
            get
            {
                return _column;
            }
        }


        public void UpdateSqlTree(SqlCeConnection conn)
        {
            String sqlStatement = "SELECT DISTINCT " + _table + " FROM " + _column + ";";

            try
            {
                SqlCeDataAdapter sqlAdapter = new SqlCeDataAdapter(sqlStatement, conn);
                treeItemDataTable.Clear();
                sqlAdapter.Fill(treeItemDataTable);
            }

            catch (SqlCeException sqlexception)
            {
                string message = sqlexception.Message;
            }

            catch (Exception ex)
            {
                string message = ex.Message;
            }
        }


        private void BuildTreeItem()
        {
            Binding SqlSelectBinding = new Binding();

            SqlSelectBinding.Source = treeItemDataTable;
            SqlSelectBinding.Mode = BindingMode.OneWay;
            this.SetBinding(TreeView.ItemsSourceProperty, SqlSelectBinding);


            Binding binding = new Binding();
            binding.Path = new PropertyPath(treeItemDataTable.Columns[0].ColumnName);
            binding.Mode = BindingMode.OneWay;
            switch (_table)
            {
                case "Rating":
                    binding.Converter = new RatingConverter();
                    break;
                case "IsoSpeed":
                    binding.Converter = new IsoSpeedConverter();
                    break;
                case "ExposureTime":
                    binding.Converter = new ExposureTimeConverter();
                    break;
                case "ExposureMode":
                    binding.Converter = new ExposureModeConverter();
                    break;
                case "LensAperture":
                    binding.Converter = new LensApertureConverter();
                    break;
                case "FocalLength":
                    binding.Converter = new FocalLengthConverter();
                    break;
                case "WhiteBalanceMode":
                    binding.Converter = new WhiteBalanceConverter();
                    break;
            }


            FrameworkElementFactory textElement = new FrameworkElementFactory(typeof(TextBlock));
            textElement.SetValue(TextBlock.ForegroundProperty, Brushes.Blue);
            textElement.SetBinding(TextBlock.TextProperty, binding);

            DataTemplate dataTemplate = new DataTemplate();
            dataTemplate.VisualTree = textElement;

            this.ItemTemplate = dataTemplate;
        }

        public SqlTreeViewItem(String header, String table, String column, SqlCeConnection conn)
            : base()
        {
            _table = table;
            _column = column;
            treeItemDataTable = new DataTable("TreeItemTable");
            UpdateSqlTree(conn);
            BuildTreeItem();

            this.Header = header;
        }
    }
}
