/*
Copyright (c) 2009, Luke Berndt
All rights reserved.
 
 * More Information: http://www.tagflo.com

Redistribution and use in source and binary forms, with or without modification, are permitted 
provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions 
  and the following disclaimer.

* Redistributions in binary form must reproduce the above copyright notice, this list of conditions 
  and the following disclaimer in the documentation and/or other materials provided with the distribution.

* Neither the name of RoboTastic Software nor the names of its contributors may be used to endorse or 
  promote products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED 
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR 
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
POSSIBILITY OF SUCH DAMAGE.
*/


using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;
using System.IO;

namespace TagFlo
{
    /// <summary>
    /// Interaction logic for PhotoScroller.xaml
    /// </summary>

    public partial class PhotoScroller : System.Windows.Controls.ListBox
    {

        public PhotoScroller()
        {
            InitializeComponent();
        }


        protected override void OnItemsChanged(NotifyCollectionChangedEventArgs e)
        {
            if (Items.Count > 0)
            {
                SelectedIndex = 0;
            }
        }


        private BitmapImage ResizeImage(string _filename, double width, double height)
        {
            byte[] buffer = File.ReadAllBytes(_filename);
            // by reading the data into an in-memory buffer,we prevent the file from being read in the UI thread -- which speeds up
            // access dramatically!
            MemoryStream mem = new MemoryStream(buffer);




            BitmapImage bi = new BitmapImage();
            bi.BeginInit();
         /*   if (width > 0)
            {
                bi.DecodePixelWidth = (int)width;
                bi.DecodePixelHeight = (int)height;
            }*/
            //bi.UriSource = new Uri(_filename); slow slow slow
            bi.StreamSource = mem;
            bi.EndInit();
            bi.Freeze();

            return bi;
            
            // if you dispose of the memory stream here, the image will be toast (burnt toast)
            // (as the dispatcher won't have run yet).
        }

        public void PhotoScroller_MouseDown(Object sender, MouseButtonEventArgs e)
        {
            Keyboard.Focus(this);
    }


        private void PhotoScroller_PreviewKeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == Key.Left)
            {
                if (SelectedIndex > 0 )
                {
                    SelectedIndex--;
                    ScrollIntoView(SelectedItem);
                    e.Handled = true;

                }
            }
            if (e.Key == Key.Right)
            {
                if (SelectedIndex < (Items.Count - 1))
                {
                    SelectedIndex++;
                    ScrollIntoView(SelectedItem);
                    e.Handled = true;
                }
            }

        }

        public void PhotoScroller_SelectionChanged(Object sender, SelectionChangedEventArgs e)
        {

            if ((SelectedItem != null) && (SelectedItem.GetType() == typeof(DataRowView)))
            {
                DataRowView drv = (DataRowView)SelectedItem;
                String path = drv["Path"].ToString();
                Image largeImage = (Image)this.Template.FindName("LargeImage", this);
                Grid grid = (Grid)this.Template.FindName("PhotoScrollerGrid", this);
               
                double h = grid.RowDefinitions[0].ActualHeight;
                    largeImage.Source = ResizeImage(path, largeImage.ActualWidth, largeImage.ActualHeight);

               
            }
        }
    }
}